# 📖 Summary
- [📖 Summary](#📖-summary)
- [💼 About {{ connector_name }}](#💼-about-{{ connector_name | lower | replace(" ", "-")}})
  - [😍 Why is it a big deal for {{ connector_name }} customers & partners?](#😍-why-is-it-a-big-deal-for-{{ connector_name | lower | replace(" ", "-")}}-customers--partners)
- [🔧 How does it work?](#🔧-how-does-it-work)
  - [📊 Data integration capabilities:](#📊-data-integration-capabilities)
  - [🧠 Artificial Intelligence capabilities:](#🧠-artificial-intelligence-capabilities)
- [🔌 Connector Actions](#🔌-connector-actions)
- [💍 Quick Start Examples](#💍-quick-start-examples)
- [🔗 Useful Links](#🔗-useful-links)
- [👏 Special Thanks](#👏-special-thanks)


# 💼 About {{ connector_name }}
{% if description %}
> {{ description }}
{% endif %}

## 😍 Why is it a big deal for {{ connector_name }} customers & partners?

This new connector will enable:
- ⚡ A Fastlane Talent & Workforce data integration for {{ connector_name }} customers & partners
- 🤖 Cutting-edge AI-powered Talent Experiences & Recruiter Experiences for {{ connector_name }} customers

#  🔧 How does it work?
## 📊 Data integration capabilities:
- ⬅️ Send Profiles data from {{ connector_name }} to a Destination of your choice.
- ➡️ Send Profiles data from a Source of your choice to {{ connector_name }}.
- ⬅️ Send Jobs data from {{ connector_name }} to a Destination of your choice.
- ➡️ Send Jobs data from a Source of your choice to {{ connector_name }}.


## 🧠 Artificial Intelligence capabilities:
- Extract, Structure, and Categorize Talent & Workforce data
- Search, Score, and Match Profiles & Jobs with our APIs and AI Widgets (**Matching Custom Tab in {{ connector_name }}**)


{% include "connector_actions.md.j2" %}


# 💍 Quick Start Examples

To make sure you can successfully run the latest versions of the example scripts, you have to **install the package from PyPi**.


To browse the examples of actions corresponding to released versions of 🤗 this connector, you just need to import the module like this :


Once the connector module is imported, you can leverage all the different actions that it offers.

For more code details checkout connector code.


# 🔗 Useful Links

- 📄Visit [{{ connector_name }}](https://www.{{ connector_name | lower | replace(" ", "-")}}.com/) to learn more.
- 💻 [Connector code](https://github.com/Riminder/hrflow-connectors/tree/master/src/hrflow_connectors/connectors/{{ connector_name | lower | replace(" ", "-")}}) on our Github.


# 👏 Special Thanks
- 💻 HrFlow.ai : XXXXX YYYYY - Software Engineer
- 🤝 {{ connector_name }} : XXXXX YYYYY - Partner Manager