CIVILITY = [
    {
        "code": 3445,
        "clientCode": "_TS_CO_Civility_Dr",
        "label": "Dr.",
        "type": "civility",
        "parentType": "",
    },
    {
        "code": 1790,
        "clientCode": "_TS_CO_Civility_Miss",
        "label": "Miss",
        "type": "civility",
        "parentType": "",
    },
    {
        "code": 280,
        "clientCode": "_TS_CO_Civility_Mr",
        "label": "Mr.",
        "type": "civility",
        "parentType": "",
    },
    {
        "code": 1789,
        "clientCode": "_TS_CO_Civility_Mrs ",
        "label": "Mrs",
        "type": "civility",
        "parentType": "",
    },
    {
        "code": 281,
        "clientCode": "_TS_CO_Civility_Mme",
        "label": "Ms.",
        "type": "civility",
        "parentType": "",
    },
    {
        "code": 3467,
        "clientCode": "_TS_CO_Civility_Other",
        "label": "Other",
        "type": "civility",
        "parentType": "",
    },
]

EDUCATIONS_REFERENTIEL = {
    "Aucun diplôme": {
        "code": 261,
        "clientCode": "_TS_etude_min_Aucun_diplome",
        "label": "No Degree",
        "type": "educationLevel",
        "parentType": "",
    },
    "BAC": {
        "code": 262,
        "clientCode": "_TS_etude_min_BAC",
        "label": "Bac",
        "type": "educationLevel",
        "parentType": "",
    },
    "BAC+2": {
        "code": 2859,
        "clientCode": "_TS_etude_min_BAC2",
        "label": "A Levels, Diploma",
        "type": "educationLevel",
        "parentType": "",
    },
    "BAC+3": {
        "code": 4028,
        "clientCode": "_TS_etude_min_Licence",
        "label": "Licence",
        "type": "educationLevel",
        "parentType": "",
    },
    "BAC+4": {
        "code": 2860,
        "clientCode": "_TS_etude_min_BAC4",
        "label": "Bachelor's Degree",
        "type": "educationLevel",
        "parentType": "",
    },
    "BAC+5": {
        "code": 266,
        "clientCode": "_TS_etude_min_BAC5",
        "label": "Master Degree",
        "type": "educationLevel",
        "parentType": "",
    },
    "CAP, BEP": {
        "code": 2858,
        "clientCode": "_TS_etude_min_CAP_BEP",
        "label": "GCSEs",
        "type": "educationLevel",
        "parentType": "",
    },
    "DOCTORAT": {
        "code": 2861,
        "clientCode": "_TS_etude_min_DOCTORAT",
        "label": "PhD, Doctorate",
        "type": "educationLevel",
        "parentType": "",
    },
    "Mastère": {
        "code": 4030,
        "clientCode": "_TS_etude_min_Mastere",
        "label": "Mastère",
        "type": "educationLevel",
        "parentType": "",
    },
}
EXPERIENCES_REFERENTIEL = {
    "Etudiant-e": "",
    "Débutant-e/première expérience": {
        "code": 274,
        "clientCode": "_TS_niveau_exp_premiere_exp",
        "label": "First experience",
        "type": "experienceLevel",
        "parentType": "",
    },
    "Supérieure à 3 ans": {
        "code": 1785,
        "clientCode": "_TS_niveau_exp_superieur_3ans ",
        "label": "More than 3 years",
        "type": "experienceLevel",
        "parentType": "",
    },
    "Supérieure à 5 ans": {
        "code": 1783,
        "clientCode": "_TS_niveau_exp_superieur_5ans",
        "label": "More than 5 years",
        "type": "experienceLevel",
        "parentType": "",
    },
    "Supérieure à 8 ans": {
        "code": 275,
        "clientCode": "_TS_niveau_exp_superieur_8ans",
        "label": "More than 8 years",
        "type": "experienceLevel",
        "parentType": "",
    },
}

CONTRACT_TYPE_REFERENTIAL = {
    "Alternance": {
        "code": 1047,
        "clientCode": "_TS_CO_Contract_Alternance",
        "label": "Apprenticeship",
        "type": "contractType",
        "parentType": "",
    },
    "CDD": {
        "code": 1045,
        "clientCode": "_TS_CO_Contract_CDD",
        "label": "Fixed Term contract",
        "type": "contractType",
        "parentType": "",
    },
    "CDI": {
        "code": 1044,
        "clientCode": "_TS_CO_Contract_CDI",
        "label": "Permanent",
        "type": "contractType",
        "parentType": "",
    },
    "CIFRE": {
        "code": 1049,
        "clientCode": "_TS_CO_Contract_CIFRE",
        "label": "CIFRE",
        "type": "contractType",
        "parentType": "",
    },
    "Stage": {
        "code": 1046,
        "clientCode": "_TS_CO_Contract_Stage",
        "label": "Internship / Student",
        "type": "contractType",
        "parentType": "",
    },
    "VIE": {
        "code": 1048,
        "clientCode": "_TS_CO_Contract_VIE",
        "label": "VIE",
        "type": "contractType",
        "parentType": "",
    },
}
